import pytest

from ..models import Invoice, InvoiceLine, InvoiceVat, refresh_invoice

pytestmark = pytest.mark.django_db


def test_invoice_create():
    invoice = Invoice()
    invoice.number = "F0000-00-0000"
    invoice.payment_mode = "cash"

    invoice.clean()

    assert invoice.total_vat_excl == 0
    assert invoice.total_vat_incl == 0

    invoice.save()

    line = InvoiceLine()
    line.invoice = invoice
    line.title = "test"
    line.description = "test"
    line.quantity = 10
    line.unit_price = 10
    line.vat_rate = 20

    line.clean()

    assert line.price_vat_excl == 100
    assert line.price_vat_only == 20
    assert line.price_vat_incl == 120

    line.save()
    refresh_invoice(sender=InvoiceLine, instance=line)

    assert InvoiceVat.objects.filter(invoice=invoice, vat_rate=20, price=20).exists()

    invoice.refresh_from_db()

    assert invoice.total_vat_excl == 100
    assert invoice.total_vat_incl == 120

    invoice.total_vat_excl = 0
    invoice.total_vat_incl = 0

    invoice.clean()

    assert invoice.total_vat_excl == 100
    assert invoice.total_vat_incl == 120



import os
from unittest.mock import patch

import pytest
from django.contrib.auth import get_user_model
from django.test import Client

# AUTOUSE FIXTURES


@pytest.fixture(autouse=True)
def env_variables(monkeypatch):
    with patch.dict(os.environ, clear=True):
        env_vars = {
            # add the test env variables here
        }
        for k, v in env_vars.items():
            monkeypatch.setenv(k, v)
        yield  # This is the magical bit which restore the environment after


# OTHER FIXTURES


@pytest.fixture
def anonymous_client():
    return Client()


@pytest.fixture
def user():
    return get_user_model().objects.create(email="user_log@example.com")


@pytest.fixture
def logged_client(user):
    client = Client()
    client.force_login(user)
    return client



"""
Django settings for Invoicer project.

Generated by 'django-admin startproject' using Django 4.2.1.

For more information on this file, see
https://docs.djangoproject.com/en/4.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.2/ref/settings/
"""

import os
from pathlib import Path

PROJECT_DIR = Path(__file__).resolve().parent
BASE_DIR = PROJECT_DIR.parent

BASE_URL = os.getenv("BASE_URL", "http://127.0.0.1:8000")

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.getenv("SECRET_KEY", "TODO_change_me_please")

DEBUG = os.getenv("DEBUG", "True") == "True"

ALLOWED_HOSTS = ["localhost", "127.0.0.1"]
if os.getenv("ALLOWED_HOSTS"):
    ALLOWED_HOSTS += os.getenv("ALLOWED_HOSTS").split(",")

# Application definition
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "hbdjinvoicing",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

if DEBUG:
    MIDDLEWARE.insert(0, "query_counter.middleware.DjangoQueryCounterMiddleware")

ROOT_URLCONF = "devproject.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [PROJECT_DIR / "templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "devproject.wsgi.application"


# Database
# https://docs.djangoproject.com/en/4.2/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": os.getenv("DB_NAME", "hbdjinvoicing"),
    }
}

if os.getenv("DB_USER"):
    DATABASES["default"]["USER"] = os.getenv("DB_USER")
if os.getenv("DB_PASSWORD"):
    DATABASES["default"]["PASSWORD"] = os.getenv("DB_PASSWORD")
if os.getenv("DB_HOST"):
    DATABASES["default"]["HOST"] = os.getenv("DB_HOST")
if os.getenv("DB_PORT"):
    DATABASES["default"]["PORT"] = os.getenv("DB_PORT")

# Password validation
# https://docs.djangoproject.com/en/4.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.2/topics/i18n/

LANGUAGE_CODE = "fr-FR"

TIME_ZONE = "Europe/Paris"

USE_I18N = True

USE_L10N = True

USE_TZ = True

DATE_INPUT_FORMATS = [
    "%d/%m/%Y",
]

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.2/howto/static-files/

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

STATIC_URL = "/static/"
STATIC_ROOT = os.getenv("STATIC_ROOT", BASE_DIR / "static")

MEDIA_URL = "/media/"
MEDIA_ROOT = os.getenv("MEDIA_ROOT", BASE_DIR / "media")

# Default primary key field type
# https://docs.djangoproject.com/en/4.2/ref/settings/#default-auto-field
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# Email settings
EMAIL_BACKEND = os.getenv(
    "EMAIL_BACKEND", "django.core.mail.backends.console.EmailBackend"
)
EMAIL_HOST = os.getenv("EMAIL_HOST")
EMAIL_PORT = int(os.getenv("EMAIL_PORT", "587"))
DEFAULT_FROM_EMAIL_NAME = os.getenv("DJANGO_DEFAULT_FROM_EMAIL_NAME")
DEFAULT_FROM_EMAIL = "{} <{}>".format(
    DEFAULT_FROM_EMAIL_NAME, os.getenv("DEFAULT_FROM_EMAIL", "extranet@example.com")
)
EMAIL_HOST_USER = os.getenv("EMAIL_HOST_USER")
EMAIL_HOST_PASSWORD = os.getenv("EMAIL_HOST_PASSWORD")
EMAIL_USE_TLS = os.getenv("EMAIL_USE_TLS", "True") == "True"
EMAIL_USE_SSL = os.getenv("EMAIL_USE_SSL", "True") == "True"

# Celery settings
CELERY_BROKER_URL = os.getenv("CELERY_BROKER_URL", "redis://127.0.0.1:6379")
CELERY_ACCEPT_CONTENT = {"application/json"}
CELERY_RESULT_SERIALIZER = "json"
CELERY_TASK_SERIALIZER = "json"
CELERY_TIMEZONE = "Europe/Paris"
CELERY_RESULT_BACKEND = "django-db"
CELERY_RESULT_EXTENDED = True

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "root": {
        "handlers": ["console"],
        "level": "WARNING",
    },
}



# Generated by Django 5.1.6 on 2025-02-25 15:43

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("hbdjinvoicing", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="invoice",
            name="customer",
        ),
        migrations.DeleteModel(
            name="Customer",
        ),
    ]



# Generated by Django 5.1.6 on 2025-02-26 16:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("hbdjinvoicing", "0003_alter_invoice_country_alter_organisation_country"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="street",
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name="organisation",
            name="street",
            field=models.TextField(),
        ),
    ]
